"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ncp_1 = require("ncp");
const path = __importStar(require("path"));
const HtmlConverter_1 = require("./converter/HtmlConverter");
const PdfConverter_1 = require("./converter/PdfConverter");
const ConversionObject_1 = require("./objects/export/ConversionObject");
const ExtensionObject_1 = require("./objects/export/ExtensionObject");
const ConverterSettingsObject_1 = require("./objects/settings/ConverterSettingsObject");
const divClassRegExp = /<div[ \t]((?:(?!class[ \t]*=[ \t]*["'])\S+[ \t]*=[ \t]*(["'])(?:\\\2|(?!\2).)*\2[ \t]*)*)class[ \t]*=[ \t]*(["'])((?:\\\3|(?!\3).)*)\3((?:(?!\/?>).|[^\/>])*)(\/?)>/gi;
const videoRegExp = /<video[ \t>]/g;
const assetsPath = '../assets';
/**
 * Allows to parse HTML code for usage of elearn.js extensions.
 * Allows to get path to extension necessary files.
 * Allows to get include strings for extension necessary files.
 */
class ExtensionManager {
    // Scan
    static scanForQuiz(html) {
        let include = false;
        html.replace(divClassRegExp, (wholeMatch, before, wrapBefore, wrap, classVal, after, closingSlash) => {
            if (classVal.match(/(?:^|\s)question(?:$|\s)/g))
                include = true;
            return "";
        });
        return include;
    }
    static scanForVideo(html) {
        let include = false;
        if (html.match(videoRegExp))
            include = true;
        return include;
    }
    static scanForClickImage(html) {
        let include = false;
        html.replace(divClassRegExp, (wholeMatch, before, wrapBefore, wrap, classVal, after, closingSlash) => {
            if (classVal.match(/(?:^|\s)clickimage(?:$|\s)/g))
                include = true;
            return "";
        });
        return include;
    }
    static scanForTimeSlider(html) {
        let include = false;
        html.replace(divClassRegExp, (wholeMatch, before, wrapBefore, wrap, classVal, after, closingSlash) => {
            if (classVal.match(/(?:^|\s)timeslider(?:$|\s)/g))
                include = true;
            return "";
        });
        return include;
    }
    /**
     * Scans the html code for elearn.js extensions.
     *
     * @param html string: the html code to be scanned
     *
     * @return ExtensionObject: including which extensions were found
     * and explicitly which were not found (true/false)
     */
    static scanHtmlForAll(html) {
        return new ExtensionObject_1.ExtensionObject({
            includeQuiz: ExtensionManager.scanForQuiz(html),
            includeElearnVideo: ExtensionManager.scanForVideo(html),
            includeClickImage: ExtensionManager.scanForClickImage(html),
            includeTimeSlider: ExtensionManager.scanForTimeSlider(html),
        });
    }
    /**
     * Scans the markdown code for elearn.js extensions.
     *
     * @param markdown string: the original markdown code to be scanned
     * @param IConverter (optional) HtmlConverter or PdfConverter:
     *  the converter to use for markdown to HTML conversion. If not given,
     *  a default HtmlConverter will be used.
     *
     * @return Promise<ExtensionObject>: including which extensions were found
     * and explicitly which were not found (true/false)
     */
    static scanMarkdownForAll(markdown, markdownConverter) {
        return __awaiter(this, void 0, void 0, function* () {
            // define converter
            if (!ExtensionManager.htmlConverter)
                ExtensionManager.htmlConverter = new HtmlConverter_1.HtmlConverter(new ConverterSettingsObject_1.ConverterSettingsObject());
            let converter = ExtensionManager.htmlConverter;
            if (markdownConverter
                && (markdownConverter instanceof HtmlConverter_1.HtmlConverter
                    || markdownConverter instanceof PdfConverter_1.PdfConverter)) {
                converter = markdownConverter;
            }
            let opts = new ConversionObject_1.ConversionObject();
            opts.bodyOnly = true;
            // convert and then scan
            let html = yield converter.toHtml(markdown, opts);
            return ExtensionManager.scanHtmlForAll(html);
        });
    }
    // Asset Strings for an HTML Export
    static getHTMLAssetStrings(includeQuiz, includeElearnVideo, includeClickImage, includeTimeSlider) {
        return `${includeQuiz ? ExtensionManager.getQuizHTMLAssetString() : ""}
                ${includeElearnVideo ? ExtensionManager.getElearnVideoHTMLAssetString() : ""}
                ${includeClickImage ? ExtensionManager.getClickImageHTMLAssetString() : ""}
                ${includeTimeSlider ? ExtensionManager.getTimeSliderHTMLAssetString() : ""}`;
    }
    // Asset Strings for a PDF Export
    static getPDFAssetStrings(includeQuiz, includeElearnVideo, includeClickImage, includeTimeSlider) {
        return `${includeQuiz ? ExtensionManager.getQuizPDFAssetString() : ""}
                ${includeElearnVideo ? ExtensionManager.getElearnVideoPDFAssetString() : ""}
                ${includeClickImage ? ExtensionManager.getClickImagePDFAssetString() : ""}
                ${includeTimeSlider ? ExtensionManager.getTimeSliderPDFAssetString() : ""}`;
    }
    /**
     * Writes the elearn.js assets to the given path.
     * @param dirPath: string - the path to write the `assets` folder to.
     * @param {ExtensionObject} opts optional options
     */
    static exportAssets(dirPath, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            let outPath = path.resolve(dirPath + "/assets/");
            let folders = [path.resolve(`${__dirname}/${assetsPath}/elearnjs/assets/`)];
            if (opts.includeQuiz)
                folders.push(ExtensionManager.getQuizAssetDir());
            if (opts.includeElearnVideo)
                folders.push(ExtensionManager.getElearnVideoAssetDir());
            if (opts.includeClickImage)
                folders.push(ExtensionManager.getClickImageAssetDir());
            if (opts.includeTimeSlider)
                folders.push(ExtensionManager.getTimeSliderAssetDir());
            return ExtensionManager.writeFolders(folders, outPath);
        });
    }
    // Locations to copy from
    static getQuizAssetDir() {
        return path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/quiz/assets/`);
    }
    static getElearnVideoAssetDir() {
        return path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/elearnvideo/assets/`);
    }
    static getClickImageAssetDir() {
        return path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/clickimage/assets/`);
    }
    static getTimeSliderAssetDir() {
        return path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/timeslider/assets/`);
    }
    static getHTMLAssetString(name) {
        return `<link rel="stylesheet" type="text/css" href="assets/css/${name}.css">
                <script type="text/javascript" src="assets/js/${name}.js"></script>`;
    }
    static getQuizHTMLAssetString() {
        return ExtensionManager.getHTMLAssetString("quiz");
    }
    static getElearnVideoHTMLAssetString() {
        return ExtensionManager.getHTMLAssetString("elearnvideo");
    }
    static getClickImageHTMLAssetString() {
        return ExtensionManager.getHTMLAssetString("clickimage");
    }
    static getTimeSliderHTMLAssetString() {
        return `<link rel="stylesheet" type="text/css" href="assets/css/timeslider.css">
                <script type="text/javascript" src="assets/js/moment.js"></script>
                <script type="text/javascript" src="assets/js/timeslider.js"></script>`;
    }
    // Asset Strings for a PDF Export
    static getQuizPDFAssetString() {
        return `<link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/quiz/assets/css/quiz.css`).replace(/\\/g, "/")}" />
                <link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/pdfAssets/css/quiz.css`).replace(/\\/g, "/")}" />
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/quiz/assets/js/quiz.js`).replace(/\\/g, "/")}"></script>`;
    }
    static getElearnVideoPDFAssetString() {
        return `<link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/elearnvideo/assets/css/elearnvideo.css`).replace(/\\/g, "/")}" />
                <link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/pdfAssets/css/elearnvideo.css`).replace(/\\/g, "/")}" />
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/elearnvideo/assets/js/elearnvideo.js`).replace(/\\/g, "/")}"></script>
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/pdfAssets/js/elearnvideo.js`).replace(/\\/g, "/")}"></script>`;
    }
    static getClickImagePDFAssetString() {
        return `<link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/clickimage/assets/css/clickimage.css`).replace(/\\/g, "/")}" />
                <link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/pdfAssets/css/clickimage.css`).replace(/\\/g, "/")}" />
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/clickimage/assets/js/clickimage.js`).replace(/\\/g, "/")}"></script>`;
    }
    static getTimeSliderPDFAssetString() {
        return `<link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/timeslider/assets/css/timeslider.css`).replace(/\\/g, "/")}" />
                <link rel="stylesheet" type="text/css"
                    href="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/pdfAssets/css/timeslider.css`).replace(/\\/g, "/")}" />
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/timeslider/assets/js/moment.js`).replace(/\\/g, "/")}"></script>
                <script type="text/javascript"
                    src="file:///${path.resolve(`${__dirname}/${assetsPath}/elearnjs/extensions/timeslider/assets/js/timeslider.js`).replace(/\\/g, "/")}"></script>`;
    }
    /**
     * Copies/writes a list of folders by their absolute paths to the outPath
     */
    static writeFolders(folders, outPath) {
        return __awaiter(this, void 0, void 0, function* () {
            // get first folder + remove from array
            for (let inPath of folders) {
                yield new Promise((res, rej) => {
                    ncp_1.ncp(inPath, outPath, (err) => {
                        if (err)
                            rej(err);
                        else
                            res();
                    });
                });
            }
        });
    }
}
exports.ExtensionManager = ExtensionManager;
