"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const Showdown = __importStar(require("showdown"));
const ExtensionManager_1 = require("../ExtensionManager");
const FileExtractor_1 = require("../FileExtractor");
const FileManager_1 = require("../FileManager");
const ConversionObject_1 = require("../objects/export/ConversionObject");
const ExtensionObject_1 = require("../objects/export/ExtensionObject");
const HtmlExportOptionObject_1 = require("../objects/export/HtmlExportOptionObject");
const InclusionObject_1 = require("../objects/export/InclusionObject");
const ConverterSettingsObject_1 = require("../objects/settings/ConverterSettingsObject");
const PromiseCounter_1 = require("../util/PromiseCounter");
const AConverter_1 = require("./AConverter");
const ShowdownExtensionManager_1 = require("./ShowdownExtensionManager");
/**
 * Converts Markdown to elearnjs HTML
 */
class HtmlConverter extends AConverter_1.AConverter {
    /**
     * Creates an HtmlConverter with specific options.
     * @param {ConverterSettingsObject} options: optional options
     */
    constructor(options) {
        super();
        this.showdownExtensions = new ShowdownExtensionManager_1.ShowdownExtensionManager();
        let showdownImprintExtensions = new ShowdownExtensionManager_1.ShowdownExtensionManager();
        this.converter = new Showdown.Converter({
            simplifiedAutoLink: true,
            excludeTrailingPunctuationFromURLs: true,
            strikethrough: true,
            tables: true,
            extensions: this.showdownExtensions.getHtmlBodyExtensions(),
        });
        this.imprintConverter = new Showdown.Converter({
            simplifiedAutoLink: true,
            excludeTrailingPunctuationFromURLs: true,
            strikethrough: true,
            tables: true,
            extensions: showdownImprintExtensions.getImprintExtensions(),
        });
        // set export defaults
        let defaults = new ConverterSettingsObject_1.ConverterSettingsObject();
        Object.keys(defaults).forEach((key) => {
            this.converter.setOption(key, defaults[key]);
        });
        // overwrite defaults with given options
        if (options) {
            this.setOptions(options);
        }
    }
    /**
     * Based on the `automaticDetection` it will create a
     * ExtensionObject which states, which extension to include.
     * @param html The HTML to scan if
     * @param opts
     */
    static fillExtensionOptions(html, opts, automaticDetection) {
        let includes = new ExtensionObject_1.ExtensionObject(opts);
        if (automaticDetection) {
            if (opts.includeQuiz === undefined)
                includes.includeQuiz = ExtensionManager_1.ExtensionManager.scanForQuiz(html);
            if (opts.includeElearnVideo === undefined)
                includes.includeElearnVideo = ExtensionManager_1.ExtensionManager.scanForVideo(html);
            if (opts.includeClickImage === undefined)
                includes.includeClickImage = ExtensionManager_1.ExtensionManager.scanForClickImage(html);
            if (opts.includeTimeSlider === undefined)
                includes.includeTimeSlider = ExtensionManager_1.ExtensionManager.scanForTimeSlider(html);
        }
        return includes;
    }
    toHtml(markdown, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            let opts = new ConversionObject_1.ConversionObject(options);
            // update converter settings from options
            self.converter.setOption("removeComments", opts.removeComments);
            self.imprintConverter.setOption("removeComments", opts.removeComments);
            let html = self.converter.makeHtml(markdown); // conversion
            if (opts.bodyOnly) {
                return html;
            }
            // create meta and imprint
            let meta = this.showdownExtensions.parseMetaData(markdown);
            let imprint = "";
            // create imprint only if explicitly inserted in markdown
            if (markdown.match(/(?:(?:^|\n)(```+|~~~+)imprint\s*?\n([\s\S]*?)\n\1|(?:^|\n)(<!--+)imprint\s*?\n([\s\S]*?)\n-->)/g)) {
                imprint = self.imprintConverter.makeHtml(markdown);
            }
            let data = yield FileManager_1.FileManager.getHtmlTemplate();
            // scan for extensions if necessary
            opts = Object.assign(opts, HtmlConverter.fillExtensionOptions(html, opts, opts.automaticExtensionDetection));
            return self.getHTMLFileContent(data, html, meta, imprint, opts);
        });
    }
    /**
     * Converts given markdown to a PDF File.
     * Certain options will specify the output.
     *
     * @param markdown: string - the markdown code
     * @param file: string - the output file path (including file name)
     * @param rootPath: string - the root path for relative paths in the file.
     * @param {HtmlExportOptionObject} options: optional options
     * @param forceOverwrite: bool - if an existing file should be overwritten.
     *
     * @return {Promise<string>} - will resolve with the path when done. (err) when an error occurred.
     */
    toFile(markdown, file, rootPath, options, forceOverwrite) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            let opts = new HtmlExportOptionObject_1.HtmlExportOptionObject(options);
            if (!file) {
                throw new Error("No output path given.");
            }
            if (fs.existsSync(file) && !forceOverwrite) {
                throw new Error("File already exists. Set `forceOverwrite` to true if you really want to overwrite the file.");
            }
            // will throw an error if cannot be opened
            yield self.tryFileOpen(file);
            let html = yield self.toHtml(markdown, opts);
            let filesToExport = [];
            // find files to export and change links
            if (opts.exportLinkedFiles) {
                let fileExtractorObject = FileExtractor_1.FileExtractor.replaceAllLinks(html);
                html = fileExtractorObject.html;
                filesToExport = filesToExport.concat(fileExtractorObject.files);
            }
            // scan for extensions if necessary
            opts = Object.assign(opts, HtmlConverter.fillExtensionOptions(html, opts, opts.automaticExtensionDetection));
            let promises = [];
            // write actual html file
            let writePromise = new Promise((resolve, reject) => {
                fs.writeFile(file, html, (err) => {
                    if (err)
                        reject(err);
                    else
                        resolve();
                });
            });
            promises.push(writePromise);
            // export assets
            if (opts.exportAssets)
                promises.push(ExtensionManager_1.ExtensionManager.exportAssets(path.dirname(file), opts));
            // export linked files
            if (opts.exportLinkedFiles)
                promises.push(FileExtractor_1.FileExtractor.extractAll(filesToExport, rootPath, path.dirname(file), 30000));
            // actual finish condition
            yield new PromiseCounter_1.PromiseCounter(promises, 30000);
            return file;
        });
    }
    /**
     * Inserts necessary elements into the HTML Template to create the
     * final fileContent.
     *
     * @param data: the content of the template_pdf.html as string
     * @param html: the converted HTML content, not the whole file, ohne what is
     *              within the elearn.js div.page (check the template)
     * @param meta: the converted meta part. HTML <scripts> and other added to
     *              the html <head>
     * @param imprint: HTML to be inserted into the elearn.js imprint
     * @param {InclusionObject} opts: optional options
     */
    getHTMLFileContent(data, html, meta, imprint, options) {
        let opts = new InclusionObject_1.InclusionObject(options);
        return data.replace(/\$\$meta\$\$/, () => meta)
            .replace(/\$\$extensions\$\$/, () => {
            return ExtensionManager_1.ExtensionManager.getHTMLAssetStrings(opts.includeQuiz, opts.includeElearnVideo, opts.includeClickImage, opts.includeTimeSlider);
        })
            .replace(/\$\$imprint\$\$/, () => imprint)
            .replace(/\$\$body\$\$/, () => html)
            .replace(/\$\$language\$\$/, () => {
            if (opts.language && opts.language !== "de") {
                return `<script>
                                eLearnJS.setLanguage("${opts.language}");
                                try { eLearnVideoJS.setLanguage("${opts.language}") } catch(e){ console.log(e) };
                                try { quizJS.setLanguage("${opts.language}") } catch(e){ console.log(e) };
                            </script>`;
            }
            return "";
        });
    }
}
exports.HtmlConverter = HtmlConverter;
