/// <reference types="node" />
import { ConversionObject } from '../objects/export/ConversionObject';
import { PdfExportOptionObject } from '../objects/export/PdfExportOptionObject';
import { PdfSettingsObject } from '../objects/settings/PdfSettingsObject';
import { AConverter } from './AConverter';
import { IConverter } from './IConverter';
import { IShowdownConverter } from './IShowdownConverter';
/**
 * Converts Markdown to elearnjs PDF
 */
export declare class PdfConverter extends AConverter implements IConverter {
    protected converter: IShowdownConverter;
    private browser?;
    private browserPromise?;
    private showdownExtensions;
    /**
     * Creates a PdfConverter with specific options.
     * @param {PdfSettingsObject} options: optional options
     */
    constructor(options?: PdfSettingsObject);
    private static fillExtensionOptions;
    setOption(opt: string, val: any): void;
    /**
     * Update multiple conversion options.
     * @param options: Object - same as in the constructor
     */
    setOptions(options: PdfSettingsObject): void;
    toHtml(markdown: string, options?: ConversionObject): Promise<string>;
    /**
     * Converts given markdown to an HTML string for an HTML to PDF conversion.
     * Certain options will specify the output.
     *
     * @deprecated use `.toHtml` instead
     *
     * @param markdown: string - the markdown code
     * @param {ConversionObject} options: optional options
     *
     * @return {Promise<string>} - will resolve with the output html, when done.
     */
    toPdfHtml(markdown: string, options?: ConversionObject): Promise<string>;
    /**
     * Converts given markdown to a PDF File.
     * Certain options will specify the output.
     *
     * @param markdown: string - the markdown code
     * @param file: string - the output file path (including file name)
     * @param rootPath: string - the root path for relative paths in the file.
     * @param {PdfExportOptionObject} options: optional options
     * @param forceOverwrite: bool - if an existing file should be overwritten.
     *
     * @return {Promise<string>} - will resolve with the path when done. (err) when an error occurred.
     */
    toFile(markdown: string, file: string, rootPath: string, options?: PdfExportOptionObject, forceOverwrite?: boolean): Promise<string>;
    /**
     * Converts given markdown to a pdf file buffer.
     * Certain options will specify the output.
     *
     * This will instantiate a chromium browser instance if not present already.
     * Changing the converter options with `setOption` or `setOptions`,
     * as well as starting another conversion process before resolving of this
     * function, might lead to unwanted behavior.
     *
     * @param markdown: string - the markdown code
     * @param file: string - the output file path (including file name)
     * @param rootPath: string - the root path for relative paths in the file.
     * @param {PdfExportOptionObject} options: optional options
     *
     * @return Promise: (stream) - will resolve when done. (err) when an error occurred.
     */
    toBuffer(markdown: string, rootPath: string, options?: PdfExportOptionObject): Promise<Buffer>;
    /**
     * Returns if the Converter has a browser instance. This instance might
     * be still in startup.
     * Will only return false if there is neither a working instance nor an
     * instance starting.
     *
     * @return whether an available or starting instance is present (true) or not (false)
     */
    hasBrowserInstance(): boolean;
    /**
     * Starts a puppeteer browser instance. Will use the
     * PDFSettingsObject.chromePath as `executablePath` if set.
     *
     * @return {IBrowser} instance of IBrowser
     */
    private initBrowser;
    /**
     * Gets a browser instance to work with.
     * Based on current settings this can be a global instance `this.browser`,
     * or a only local instance.
     * Makes sure there are never multiple browser instances created, when
     * the Option `keepChromeAlive` is set to true.
     *
     * @return the puppeteer browser instance
     */
    private getBrowserInstance;
    /**
     * Checks if the given browser references the global browser.
     * @param browser the browser to check
     *
     * @return true if the reference is equal to the global browser
     */
    private isGlobalBrowser;
    /**
     * Closes the current browser instance and resets variables.
     * Resolves when done.
     */
    private closeGlobalBrowser;
    /**
     * Closes the given browser instance.
     * Resolves when done.
     */
    private closeBrowser;
    /**
     * Adds a lock for a specific process id (IBrowser.pid)
     * @param pid the id of the browsers process.
     */
    private lockBrowser;
    /**
     * Removes a lock for a specific process id (IBrowser.pid)
     * @param pid the id of the browsers process.
     */
    private unlockBrowser;
    /**
     * Returns if the browser process is locked or not.
     * @param pid the id of the browsers process.
     */
    private isLocked;
    /**
     * Inserts necessary elements into the PDF Template to create the
     * final fileContent.
     *
     * @param data: the content of the template_pdf.html as string
     * @param html: the base HTML as generated HTML Body of the file
     * @param meta: additional header elements for the HTML file
     * @param opts: InclusionObject
     */
    private getPDFFileContent;
    /**
     * Generates the PDF output options for the used node-html-pdf package.
     * @param filePath path to the currently opened file, necessary to link
     *                 included assets.
     * @param renderDelay (optional) delay of rendering by the package in ms.
     */
    private getPdfOutputOptions;
    private wrapHeaderFooter;
    private convertKeywords;
    private getDefaultHeader;
    private getDefaultFooter;
    /**
     * Calculates the new page margins based on original margins and the
     * given headerHeight and footerHeight.
     *
     * @param margin the Puppeteer.PDFOptions.margin object.
     * @param headerHeight the header height string with units "px", "in", "cm" or "mm"
     * @param footerHeight the footer height string with units "px", "in", "cm" or "mm"
     */
    private calculateMargins;
    private convertUnitsToMMFloat;
}
