"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const Showdown = __importStar(require("showdown"));
const underscore_1 = __importDefault(require("underscore"));
const ExtensionManager_1 = require("../ExtensionManager");
const FileManager_1 = require("../FileManager");
const ConversionObject_1 = require("../objects/export/ConversionObject");
const InclusionObject_1 = require("../objects/export/InclusionObject");
const PdfExportOptionObject_1 = require("../objects/export/PdfExportOptionObject");
const PdfSettingsObject_1 = require("../objects/settings/PdfSettingsObject");
const AConverter_1 = require("./AConverter");
const ShowdownExtensionManager_1 = require("./ShowdownExtensionManager");
const assetsPath = '../../assets';
/**
 * Converts Markdown to elearnjs PDF
 */
class PdfConverter extends AConverter_1.AConverter {
    /**
     * Creates a PdfConverter with specific options.
     * @param {PdfSettingsObject} options: optional options
     */
    constructor(options) {
        super();
        this.showdownExtensions = new ShowdownExtensionManager_1.ShowdownExtensionManager();
        this.converter = new Showdown.Converter({
            simplifiedAutoLink: true,
            excludeTrailingPunctuationFromURLs: true,
            strikethrough: true,
            tables: true,
            extensions: this.showdownExtensions.getPdfBodyExtensions(),
        });
        // set export defaults
        let defaults = new PdfSettingsObject_1.PdfSettingsObject();
        Object.keys(defaults).forEach((key) => {
            this.converter.setOption(key, defaults[key]);
        });
        // overwrite defaults with given options
        if (options) {
            this.setOptions(options);
        }
    }
    static fillExtensionOptions(html, opts) {
        if (opts.automaticExtensionDetection) {
            if (opts.includeQuiz === undefined)
                opts.includeQuiz = ExtensionManager_1.ExtensionManager.scanForQuiz(html);
            if (opts.includeElearnVideo === undefined)
                opts.includeElearnVideo = ExtensionManager_1.ExtensionManager.scanForVideo(html);
            if (opts.includeClickImage === undefined)
                opts.includeClickImage = ExtensionManager_1.ExtensionManager.scanForClickImage(html);
            if (opts.includeTimeSlider === undefined)
                opts.includeTimeSlider = ExtensionManager_1.ExtensionManager.scanForTimeSlider(html);
        }
        return opts;
    }
    setOption(opt, val) {
        // reset browser, if browser specific options change
        if ((opt === "chromePath" && this.getOption("chromePath") !== val)
            || (opt === "puppeteerOptions" && !underscore_1.default.isEqual(this.getOption("puppeteerOptions"), val))
            || (opt === "keepChromeAlive" && val === false)) {
            this.closeGlobalBrowser().then();
        }
        super.setOption(opt, val);
    }
    /**
     * Update multiple conversion options.
     * @param options: Object - same as in the constructor
     */
    setOptions(options) {
        let checkedObject = new PdfSettingsObject_1.PdfSettingsObject(options);
        let inputKeys = Object.keys(options);
        Object.keys(checkedObject).forEach((key) => {
            // always check if the option is in the input object
            // this way no other option keys are set to default again.
            if (inputKeys.indexOf(key) >= 0)
                this.setOption(key, checkedObject[key]);
        });
    }
    toHtml(markdown, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            let opts = new ConversionObject_1.ConversionObject(options);
            // update converter settings from options
            self.converter.setOption("removeComments", opts.removeComments);
            let html = self.converter.makeHtml(markdown); // conversion
            if (opts.bodyOnly) {
                return html;
            }
            // create meta and imprint
            let meta = this.showdownExtensions.parseMetaData(markdown);
            let data = yield FileManager_1.FileManager.getPdfTemplate();
            // scan for extensions if necessary
            opts = Object.assign(opts, PdfConverter.fillExtensionOptions(html, opts));
            return self.getPDFFileContent(data, html, meta, opts);
        });
    }
    /**
     * Converts given markdown to an HTML string for an HTML to PDF conversion.
     * Certain options will specify the output.
     *
     * @deprecated use `.toHtml` instead
     *
     * @param markdown: string - the markdown code
     * @param {ConversionObject} options: optional options
     *
     * @return {Promise<string>} - will resolve with the output html, when done.
     */
    toPdfHtml(markdown, options) {
        return this.toHtml(markdown, options);
    }
    /**
     * Converts given markdown to a PDF File.
     * Certain options will specify the output.
     *
     * @param markdown: string - the markdown code
     * @param file: string - the output file path (including file name)
     * @param rootPath: string - the root path for relative paths in the file.
     * @param {PdfExportOptionObject} options: optional options
     * @param forceOverwrite: bool - if an existing file should be overwritten.
     *
     * @return {Promise<string>} - will resolve with the path when done. (err) when an error occurred.
     */
    toFile(markdown, file, rootPath, options, forceOverwrite) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            if (!file) {
                throw new Error("No output path given.");
            }
            if (fs.existsSync(file) && !forceOverwrite) {
                throw new Error("File already exists. Set `forceOverwrite` to true if you really want to overwrite the file.");
            }
            // will throw an error if cannot be opened
            yield self.tryFileOpen(file);
            let buffer = yield self.toBuffer(markdown, rootPath, options);
            fs.writeFileSync(file, buffer);
            return file;
        });
    }
    /**
     * Converts given markdown to a pdf file buffer.
     * Certain options will specify the output.
     *
     * This will instantiate a chromium browser instance if not present already.
     * Changing the converter options with `setOption` or `setOptions`,
     * as well as starting another conversion process before resolving of this
     * function, might lead to unwanted behavior.
     *
     * @param markdown: string - the markdown code
     * @param file: string - the output file path (including file name)
     * @param rootPath: string - the root path for relative paths in the file.
     * @param {PdfExportOptionObject} options: optional options
     *
     * @return Promise: (stream) - will resolve when done. (err) when an error occurred.
     */
    toBuffer(markdown, rootPath, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            let opts = new PdfExportOptionObject_1.PdfExportOptionObject(options);
            let html = yield self.toHtml(markdown, opts);
            let browser = yield self.getBrowserInstance();
            // add lock
            self.lockBrowser(browser);
            const page = yield browser.newPage();
            const tmpFile = path.join(rootPath, `.tmpPdfExport_${new Date().getTime()}_${Math.floor(Math.random() * 10000)}.html`);
            fs.writeFileSync(tmpFile, html, "UTF8");
            let buffer;
            try {
                // goto root path
                yield page.goto('file:///' + tmpFile.replace('\\', '/'));
                // render delay
                if (opts && opts.renderDelay) {
                    yield new Promise((res, rej) => {
                        setTimeout(res, opts.renderDelay);
                    });
                }
                buffer = yield page.pdf(self.getPdfOutputOptions());
                // close the page only if the browser is not closed anyway
                // to prevent race conditions: page.close() / browser.close() might
                // concur even though await is given
                if (self.getOption("keepChromeAlive") && self.isGlobalBrowser(browser)) {
                    yield page.close();
                }
                // remove lock
                self.unlockBrowser(browser);
                // close local browser
                if (!self.isGlobalBrowser(browser)) {
                    yield self.closeBrowser(browser);
                }
                // close global browser
                else if (!self.getOption("keepChromeAlive")) {
                    yield self.closeGlobalBrowser();
                }
                // remove tmp file
                fs.unlinkSync(tmpFile);
            }
            catch (err) {
                // remove tmp file
                fs.unlinkSync(tmpFile);
                throw err;
            }
            return buffer;
        });
    }
    /**
     * Returns if the Converter has a browser instance. This instance might
     * be still in startup.
     * Will only return false if there is neither a working instance nor an
     * instance starting.
     *
     * @return whether an available or starting instance is present (true) or not (false)
     */
    hasBrowserInstance() {
        return this.browser !== undefined || this.browserPromise !== undefined;
    }
    /**
     * Starts a puppeteer browser instance. Will use the
     * PDFSettingsObject.chromePath as `executablePath` if set.
     *
     * @return {IBrowser} instance of IBrowser
     */
    initBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.getOption('puppeteerOptions') || {};
            if (this.getOption('chromePath') !== undefined) {
                options.executablePath = this.getOption('chromePath');
            }
            const browser = yield puppeteer_1.default.launch(options);
            browser.pid = (yield browser.process()).pid;
            browser.locks = 0;
            console.log("markdown-elearnjs: Initialized new chromium browser. Process ID", browser.pid);
            return browser;
        });
    }
    /**
     * Gets a browser instance to work with.
     * Based on current settings this can be a global instance `this.browser`,
     * or a only local instance.
     * Makes sure there are never multiple browser instances created, when
     * the Option `keepChromeAlive` is set to true.
     *
     * @return the puppeteer browser instance
     */
    getBrowserInstance() {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            let browser = self.browser;
            // init browser if not initiated
            if (browser === undefined) {
                let promise;
                // use promise in progress
                if (self.getOption("keepChromeAlive") && self.browserPromise !== undefined) {
                    promise = self.browserPromise;
                }
                // create new browser instance
                else {
                    promise = self.initBrowser();
                    if (self.getOption("keepChromeAlive")) {
                        self.browserPromise = promise;
                    }
                }
                try {
                    browser = yield promise;
                }
                catch (err) {
                    self.browserPromise = undefined;
                    throw err;
                }
                // update global instance
                if (self.getOption("keepChromeAlive")
                    && self.browser === undefined
                    && self.browserPromise === promise) {
                    self.browser = browser;
                }
                // remove reference to resolved promise
                if (self.browserPromise !== undefined) {
                    self.browserPromise = undefined;
                }
            }
            return browser;
        });
    }
    /**
     * Checks if the given browser references the global browser.
     * @param browser the browser to check
     *
     * @return true if the reference is equal to the global browser
     */
    isGlobalBrowser(browser) {
        return browser === this.browser;
    }
    /**
     * Closes the current browser instance and resets variables.
     * Resolves when done.
     */
    closeGlobalBrowser() {
        return __awaiter(this, void 0, void 0, function* () {
            let browser = this.browser; // keep locally for later closing
            // remove global references
            this.browser = undefined;
            this.browserPromise = undefined;
            if (browser !== undefined)
                yield this.closeBrowser(browser);
        });
    }
    /**
     * Closes the given browser instance.
     * Resolves when done.
     */
    closeBrowser(browser) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isGlobalBrowser(browser))
                throw new Error("Cannot close global browser with `closeBrowser`. Please use `closeGlobalBrowser`.");
            // check lock right before closing
            if (browser !== undefined && !this.isLocked(browser)) {
                yield browser.close();
                console.log("markdown-elearnjs: Closed chromium browser successfully. Process ID", browser.pid);
            }
        });
    }
    /**
     * Adds a lock for a specific process id (IBrowser.pid)
     * @param pid the id of the browsers process.
     */
    lockBrowser(browser) {
        browser.locks++;
    }
    /**
     * Removes a lock for a specific process id (IBrowser.pid)
     * @param pid the id of the browsers process.
     */
    unlockBrowser(browser) {
        browser.locks--;
    }
    /**
     * Returns if the browser process is locked or not.
     * @param pid the id of the browsers process.
     */
    isLocked(browser) {
        return browser.locks > 0;
    }
    /**
     * Inserts necessary elements into the PDF Template to create the
     * final fileContent.
     *
     * @param data: the content of the template_pdf.html as string
     * @param html: the base HTML as generated HTML Body of the file
     * @param meta: additional header elements for the HTML file
     * @param opts: InclusionObject
     */
    getPDFFileContent(data, html, meta, options) {
        const self = this;
        let opts = new InclusionObject_1.InclusionObject(options);
        let zoom = `<style>html {zoom: ${self.converter.getOption('contentZoom')}}</style>`;
        let customStyleFile = self.converter.getOption('customStyleFile');
        let customStyle = "";
        if (customStyleFile && customStyleFile.length > 0 && fs.existsSync(path.resolve(customStyleFile))) {
            customStyleFile = "file:///" + path.resolve(customStyleFile).replace(/\\/g, "/");
            customStyle = `<link rel="stylesheet" type="text/css" href="${customStyleFile}">`;
        }
        return data.replace(/\$\$meta\$\$/, () => meta)
            .replace(/\$\$extensions\$\$/, () => {
            return ExtensionManager_1.ExtensionManager.getPDFAssetStrings(opts.includeQuiz, opts.includeElearnVideo, opts.includeClickImage, opts.includeTimeSlider);
        })
            .replace(/\$\$zoom\$\$/, () => zoom)
            .replace(/\$\$custom_style\$\$/, () => customStyle)
            .replace(/\$\$body\$\$/, () => html)
            .replace(/\$\$language\$\$/, () => {
            if (opts.language && opts.language !== "de") {
                return `<script>
                                eLearnJS.setLanguage("${opts.language}");
                                try { eLearnVideoJS.setLanguage("${opts.language}") } catch(e){ console.log(e) };
                                try { quizJS.setLanguage("${opts.language}") } catch(e){ console.log(e) };
                            </script>`;
            }
            return "";
        })
            .replace(/\$\$assetspath\$\$/g, () => "file:///" + path.resolve(`${__dirname}/${assetsPath}/elearnjs/`).replace(/\\/g, "/"));
    }
    /**
     * Generates the PDF output options for the used node-html-pdf package.
     * @param filePath path to the currently opened file, necessary to link
     *                 included assets.
     * @param renderDelay (optional) delay of rendering by the package in ms.
     */
    getPdfOutputOptions() {
        const self = this;
        // header and footer
        let header = self.converter.getOption('customHeader');
        if (!header)
            header = self.getDefaultHeader();
        let footer = self.converter.getOption('customFooter');
        if (!footer)
            footer = self.getDefaultFooter();
        header = this.wrapHeaderFooter(header);
        footer = this.wrapHeaderFooter(footer);
        // legacy support
        header = this.convertKeywords(header);
        footer = this.convertKeywords(footer);
        let opts = {
            format: "A4",
            margin: {
                top: "18mm",
                right: "23mm",
                bottom: "25mm",
                left: "23mm",
            },
            displayHeaderFooter: true,
            headerTemplate: header,
            footerTemplate: footer,
        };
        opts.margin = this.calculateMargins(opts.margin, self.converter.getOption('headerHeight'), self.converter.getOption('footerHeight'));
        return opts;
    }
    wrapHeaderFooter(html) {
        return `<div style="position: relative; box-sizing: border-box; font-size: 7pt; width: 100%;">${html}</div>`;
    }
    convertKeywords(html) {
        html = html.replace(/\{\{page\}\}/g, '<span class="pageNumber"></span>');
        html = html.replace(/\{\{pages\}\}/g, '<span class="totalPages"></span>');
        return html;
    }
    getDefaultHeader() {
        return `<div></div>`;
    }
    getDefaultFooter() {
        return `<div style="color: #666; text-align: right;
            padding: 0 16mm 12mm;">{{page}}</div>`;
    }
    /**
     * Calculates the new page margins based on original margins and the
     * given headerHeight and footerHeight.
     *
     * @param margin the Puppeteer.PDFOptions.margin object.
     * @param headerHeight the header height string with units "px", "in", "cm" or "mm"
     * @param footerHeight the footer height string with units "px", "in", "cm" or "mm"
     */
    calculateMargins(margin, headerHeight, footerHeight) {
        if (headerHeight) {
            try {
                let headerMM = this.convertUnitsToMMFloat(headerHeight);
                let topMM = this.convertUnitsToMMFloat(margin.top ? margin.top.toString() : "0mm");
                if (headerMM !== undefined && topMM !== undefined)
                    margin.top = (headerMM + topMM) + "mm";
            }
            catch (err) {
                // ignore
            }
        }
        if (footerHeight) {
            try {
                let footerMM = this.convertUnitsToMMFloat(footerHeight);
                let bottomMM = this.convertUnitsToMMFloat(margin.bottom ? margin.bottom.toString() : "0mm");
                if (footerMM !== undefined && bottomMM !== undefined)
                    margin.bottom = (footerMM + bottomMM) + "mm";
            }
            catch (err) {
                // ignore
            }
        }
        return margin;
    }
    convertUnitsToMMFloat(value) {
        let mm;
        if (value.match(/\d+(\.\d+)?px/)) {
            let px = parseFloat(value.replace(/\D+$/, ''));
            mm = px * 0.264583;
        }
        else if (value.match(/\d+(\.\d+)?in/)) {
            let inch = parseFloat(value.replace(/\D+$/, ''));
            mm = inch * 25.4;
        }
        else if (value.match(/\d+(\.\d+)?cm/)) {
            let cm = parseFloat(value.replace(/\D+$/, ''));
            mm = cm * 10;
        }
        else if (value.match(/\d+(\.\d+)?mm/)) {
            mm = parseFloat(value.replace(/\D+$/, ''));
        }
        return mm;
    }
}
exports.PdfConverter = PdfConverter;
